@@ -1,22 +1,25 @@
+from __future__ import annotations
+from typing import Any
+
 import re
 from datetime import datetime
 
 import shutil
 
-from PIL import Image
-import ascii_magic
+from PIL import Image  # type: ignore
+import ascii_magic  # type: ignore
 
 
-def robustTimeParse(timeString):
-    timestring = timeString.replace('Z', '+00:00')
+def robustTimeParse(timeString: str) -> datetime:
+    timeString = timeString.replace('Z', '+00:00')
     match = re.search(
-        r'^(?P<start>\d{4}-\d{2}-\d{2}T\d{2}:\d{2}:\d{2}\.)(?P<fractions>\d+)(?P<end>\+\d{2}:\d{2})$', timestring)
+        r'^(?P<start>\d{4}-\d{2}-\d{2}T\d{2}:\d{2}:\d{2}\.)(?P<fractions>\d+)(?P<end>\+\d{2}:\d{2})$', timeString)
     if match:
-        timestring = match.group('start') + match.group('fractions').ljust(6, "0") + match.group('end')
-    return datetime.fromisoformat(timestring).replace(microsecond=0)
+        timeString = match.group('start') + match.group('fractions').ljust(6, "0") + match.group('end')
+    return datetime.fromisoformat(timeString).replace(microsecond=0)
 
 
-def toBool(value):
+def toBool(value: Any) -> bool:
     if value in [True, 'True', 'true', 'yes']:
         return True
     if value in [False, 'False', 'false', 'no']:
@@ -24,7 +27,7 @@ def toBool(value):
     raise ValueError('Not a valid boolean value (True/False)')
 
 
-def imgToASCIIArt(img, columns=0, mode=ascii_magic.Modes.TERMINAL):
+def imgToASCIIArt(img: Image, columns: int = 0, mode: ascii_magic.Modes = ascii_magic.Modes.TERMINAL) -> str:
     bbox = img.getbbox()
 
     # Crop the image to the contents of the bounding box
